@@ -2,10 +2,8 @@ use std::any::Any;
 use std::cell::RefCell;
 use std::collections::HashSet;
 use std::hash::{Hash, Hasher};
-use std::mem;
-use std::ptr;
-use std::rc::Rc;
-use std::rc::Weak;
+use std::rc::{Rc, Weak};
+use std::{mem, ptr};
 
 use smallvec::SmallVec;
 
@@ -13,22 +11,32 @@ use super::*;
 
 /// The number of effects that are allocated on the stack before resorting to heap allocation in
 /// [`ReactiveScope`].
-const REACTIVE_SCOPE_EFFECTS_STACK_CAPACITY: usize = 4;
+const REACTIVE_SCOPE_EFFECTS_STACK_CAPACITY: usize = 2;
 
 /// Initial capacity for [`CONTEXTS`].
 const CONTEXTS_INITIAL_CAPACITY: usize = 10;
+/// Initial capacity for [`SCOPES`].
+const SCOPES_INITIAL_CAPACITY: usize = 4;
 
 thread_local! {
     /// Context of the effect that is currently running. `None` if no effect is running.
     ///
-    /// This is an array of callbacks that, when called, will add the a `Signal` to the `handle` in the argument.
-    /// The callbacks return another callback which will unsubscribe the `handle` from the `Signal`.
-    pub(super) static CONTEXTS: RefCell<Vec<Weak<RefCell<Option<Running>>>>> = RefCell::new(Vec::with_capacity(CONTEXTS_INITIAL_CAPACITY));
-    pub(super) static SCOPE: RefCell<Option<ReactiveScope>> = RefCell::new(None);
+    /// The [`Running`] contains an array of callbacks that, when called, will add the a `Signal` to
+    /// the `handle` in the argument. The callbacks return another callback which will unsubscribe the
+    /// `handle` from the `Signal`.
+    pub(super) static CONTEXTS: RefCell<Vec<Weak<RefCell<Option<Running>>>>> =
+        RefCell::new(Vec::with_capacity(CONTEXTS_INITIAL_CAPACITY));
+    /// Explicit stack of [`ReactiveScope`]s.
+    pub(super) static SCOPES: RefCell<Vec<ReactiveScope>> =
+        RefCell::new(Vec::with_capacity(SCOPES_INITIAL_CAPACITY));
 }
 
 /// State of the current running effect.
 /// When the state is dropped, all dependencies are removed (both links and backlinks).
+///
+/// The difference between [`Running`] and [`ReactiveScope`] is that [`Running`] is used for
+/// dependency tracking whereas [`ReactiveScope`] is used for resource cleanup. Each [`Running`]
+/// contains a [`ReactiveScope`].
 pub(super) struct Running {
     /// Callback to run when the effect is recreated.
     pub(super) execute: Rc<RefCell<dyn FnMut()>>,
@@ -43,9 +51,7 @@ impl Running {
     /// Should be called when re-executing an effect to recreate all dependencies.
     fn clear_dependencies(&mut self) {
         for dependency in &self.dependencies {
-            dependency
-                .signal()
-                .unsubscribe(&Callback(Rc::downgrade(&self.execute)));
+            dependency.signal().unsubscribe(Rc::as_ptr(&self.execute));
         }
         self.dependencies.clear();
     }
@@ -54,12 +60,18 @@ impl Running {
 /// Owns the effects created in the current reactive scope.
 /// The effects are dropped and the cleanup callbacks are called when the [`ReactiveScope`] is
 /// dropped.
+///
+/// A new [`ReactiveScope`] is usually created with [`create_root`]. A new [`ReactiveScope`] is also
+/// created when a new effect is created with [`create_effect`] and other reactive utilities that
+/// call it under the hood.
 #[derive(Default)]
 pub struct ReactiveScope {
     /// Effects created in this scope.
     effects: SmallVec<[Rc<RefCell<Option<Running>>>; REACTIVE_SCOPE_EFFECTS_STACK_CAPACITY]>,
     /// Callbacks to call when the scope is dropped.
     cleanup: Vec<Box<dyn FnOnce()>>,
+    /// Contexts created in this scope.
+    pub(super) context: Option<Box<dyn ContextAny>>,
 }
 
 impl ReactiveScope {
@@ -93,38 +105,22 @@ impl Drop for ReactiveScope {
     }
 }
 
+pub(super) type CallbackPtr = *const RefCell<dyn FnMut()>;
+
 #[derive(Clone)]
 pub(super) struct Callback(pub(super) Weak<RefCell<dyn FnMut()>>);
 
 impl Callback {
-    #[track_caller]
-    #[must_use = "returned value must be manually called"]
-    pub fn callback(&self) -> Rc<RefCell<dyn FnMut()>> {
-        self.try_callback().expect("callback is not valid anymore")
-    }
-
     #[must_use = "returned value must be manually called"]
     pub fn try_callback(&self) -> Option<Rc<RefCell<dyn FnMut()>>> {
         self.0.upgrade()
     }
-}
 
-impl Hash for Callback {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        Rc::as_ptr(&self.callback()).hash(state);
+    pub fn as_ptr(&self) -> CallbackPtr {
+        Weak::as_ptr(&self.0)
     }
 }
 
-impl PartialEq for Callback {
-    fn eq(&self, other: &Self) -> bool {
-        ptr::eq::<()>(
-            Rc::as_ptr(&self.callback()).cast(),
-            Rc::as_ptr(&other.callback()).cast(),
-        )
-    }
-}
-impl Eq for Callback {}
-
 /// A strong backlink to a [`Signal`] for any type `T`.
 #[derive(Clone)]
 pub(super) struct Dependency(pub(super) Rc<dyn AnySignalInner>);
@@ -210,12 +206,15 @@ pub fn create_effect_initial<R: 'static>(
                         running.borrow_mut().as_mut().unwrap().scope = new_scope;
                     }
 
+                    let running = running.borrow();
+                    let running = running.as_ref().unwrap();
+
                     // Attach new dependencies.
-                    for dependency in &running.borrow().as_ref().unwrap().dependencies {
+                    for dependency in &running.dependencies {
                         dependency.signal().subscribe(Callback(Rc::downgrade(
                             // Reference the same closure we are in right now.
                             // When the dependency changes, this closure will be called again.
-                            &running.borrow().as_ref().unwrap().execute,
+                            &running.execute,
                         )));
                     }
 
@@ -242,11 +241,11 @@ pub fn create_effect_initial<R: 'static>(
             "Running should be owned exclusively by ReactiveScope"
         );
 
-        SCOPE.with(|scope| {
-            if scope.borrow().is_some() {
+        SCOPES.with(|scope| {
+            if scope.borrow().last().is_some() {
                 scope
                     .borrow_mut()
-                    .as_mut()
+                    .last_mut()
                     .unwrap()
                     .add_effect_state(running);
             } else {
@@ -366,7 +365,9 @@ where
     })
 }
 
-/// Run the passed closure inside an untracked scope.
+/// Run the passed closure inside an untracked dependency scope.
+///
+/// This does **NOT** create a new [`ReactiveScope`].
 ///
 /// See also [`StateHandle::get_untracked()`].
 ///
@@ -428,11 +429,11 @@ pub fn untrack<T>(f: impl FnOnce() -> T) -> T {
 /// assert_eq!(*cleanup_called.get(), true);
 /// ```
 pub fn on_cleanup(f: impl FnOnce() + 'static) {
-    SCOPE.with(|scope| {
-        if scope.borrow().is_some() {
+    SCOPES.with(|scope| {
+        if scope.borrow().last().is_some() {
             scope
                 .borrow_mut()
-                .as_mut()
+                .last_mut()
                 .unwrap()
                 .add_cleanup(Box::new(f));
         } else {
